/*
 * Copyright (c) 2002-2005 The Regents of The University of Michigan
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer;
 * redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution;
 * neither the name of the copyright holders nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Authors: Dave Greene
 *          Nathan Binkert
 */

#include <iostream>
#include <algorithm>

#include "sized.hh"
#include <queue>
#include <typeinfo>

template<typename C>
void print(C &cont)
{
  std::cout << std::endl;
  std::cout << "Printing " << typeid(cont).name() << std::endl;
  while (!cont.empty()) {
    std::cout << cont.front() << " ";
    cont.pop();
  }
  std::cout << std::endl;
}

int main(void)
{
  sized<std::queue<int>, sized_error_policy<std::queue<int> > >
    error_queue(10);
  sized<std::queue<int>, sized_drop_policy<std::queue<int> > >
    drop_queue(5);

  for (int i = 0; i < 10; ++i) {
    error_queue.push(i);
  }

  for (int i = 0; i < 3; ++i) {
    drop_queue.push(i);
  }

  print(error_queue);
  print(drop_queue);

  return(0);
}
